class ChapterReaderUtility {

  Words words = new Words();
  ArrayList chapterList = new ArrayList();
  int XSIZE = 43;
  int YSIZE = 28;

  void initFromCompressedBible() {
    println("START initFromCompressedBible");
    CompressedBible compressedBible = new CompressedBible();
    words = compressedBible.words;
    chapterList = compressedBible.chapterList;
    println("END initFromCompressedBible");
  }

  Chapter getChapterByCoordinates(int xc, int yc) {

    PVector mousePosition = new PVector(xc, yc);

    for (int c=0; c<chapterList.size(); c++) {
      Chapter chapter = (Chapter) chapterList.get(c);
      PVector checkLocation = new PVector(chapter.location.x + (chapter.iWidth / 2), 
      chapter.location.y + (chapter.iHeight / 2));

      if (checkLocation.dist(mousePosition) < chapter.iWidth) {
        if (chapter.marked) {
          chapter.marked = false;
          return null;
        } 
        else { 
          chapter.marked = true;
          return chapter;
        }
      }
    }

    return null;
  }

  void saveCsvData(String file) {

    try {
      BufferedWriter writer = new BufferedWriter(new FileWriter(file, false));
      
      String line = "book,chapter,x,y,red,green,blue,overexpose,marked";
      println(line);
      writer.write(line);
      writer.newLine();
      
      for (int x=0; x<chapterList.size(); x++) {
        Chapter chapter = (Chapter) chapterList.get(x);

        line = chapter.book + "," + chapter.chapterNumber + "," + chapter.location.x + "," + chapter.location.y +
          "," + red(chapter.chapterColor) + "," + green(chapter.chapterColor) + "," + blue(chapter.chapterColor) + "," +
          chapter.overexpose + "," + chapter.marked;

        println(line);
        writer.write(line);
        writer.newLine();
      }

      writer.flush();
      writer.close();
    } 
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * retrieve a 2 dimensional array of chapters
   */
  void getChaptersByKeyword(String keyword) {

    int c = 0;

    int iWidth = (width / XSIZE) - 6;
    int iHeight = (height / YSIZE) -8;

    int actualX = 50;
    int actualY = 50;

    //Clear chapterList from overexposing and other stuff
    for (int x=0; x<chapterList.size(); x++) {
      Chapter chapter = (Chapter) chapterList.get(x);
      chapter.overexpose = 0;
      chapter.marked = false;
    }

    for (int y=0; y<YSIZE; y++) {
      for (int x=0; x<XSIZE; x++) {

        if (c >= chapterList.size()) break;
        Chapter chapter = (Chapter) chapterList.get(c);

        if (chapter == null) break;

        chapter.iWidth = iWidth;
        chapter.iHeight = iHeight;
        PVector location = new PVector(actualX, actualY);
        chapter.location = location;

        println(c + " " + chapter.book + " " + chapter.chapterNumber);

        //COLOR
        int [] colors = getColorsByKeyWords(keyword, chapter);

        if (colors.length == 3) {
          chapter.chapterColor = color(colors[0], colors[1], colors[2]);
        } 
        else if (colors.length == 3) {
          chapter.chapterColor = color(colors[0], colors[1], 0);
        } 
        else if (colors.length == 1) {
          int actualColor = colors[0];
          chapter.chapterColor = color(actualColor, actualColor, actualColor);
        }

        actualX += iWidth + 4;

        c += 1;
      }

      actualX = 50;
      actualY += iHeight + 4;
    }


    //TODO: reorder chapters and assign them the proper color and position;
    // velocity depends on the presence of keywords
  }

  int [] getColorsByKeyWords(String keywords, Chapter chapter) {

    int colors[] = new int[3];
    String [] keywordArray = split(keywords, ",");

    if (keywordArray.length == 3) {

      Word word = words.getWord(keywordArray[0]);
      colors[0] = getColorByKeyword(keywordArray[0], chapter, word);

      word = words.getWord(keywordArray[1]);
      colors[1] = getColorByKeyword(keywordArray[1], chapter, word);

      word = words.getWord(keywordArray[2]);
      colors[2] = getColorByKeyword(keywordArray[2], chapter, word);
    } 
    else if (keywordArray.length == 2) {

      Word word = words.getWord(keywordArray[0]);
      colors[0] = getColorByKeyword(keywordArray[0], chapter, word);

      word = words.getWord(keywordArray[1]);
      colors[1] = getColorByKeyword(keywordArray[1], chapter, word);
    }
    else {
      colors = new int[1];
      Word word = words.getWord(keywordArray[0]);
      colors[0] = getColorByKeyword(keywordArray[0], chapter, word);
    }

    return colors;
  }

  int getColorByKeyword(String keyword, Chapter chapter, Word word) {

    Word actualWord = chapter.words.getWord(keyword);

    if (actualWord == null) return 0;

    int actualColor = 0;

    if (word.count > 255) {
      actualColor = (word.count / actualWord.count) / (word.count / 255);
    } 
    else if (actualWord.count > 0) {
      actualColor = (word.count / actualWord.count);
    }

    actualColor = 255 - actualColor;

    if (actualColor < 0) {
      actualColor = 0;
    }

    if (actualColor > 200) {
      chapter.overexpose = actualColor - 200;
      actualColor = 255;
    }

    return actualColor;
  }

  void drawAllChapters() {
    int c = 0;

    for (int x=0; x<XSIZE; x++) {
      for (int y=0; y<YSIZE; y++) {
        if (c >= chapterList.size()) break;
        Chapter chapter = (Chapter) chapterList.get(c);

        if (chapter == null) break;

        chapter.draw();
        c += 1;
      }
    }
  }
}

